/*
 * LeStat - Analizador estadistico de frecuencia literal
 *
 * Calcula la frequencia (porcentaje) de caracteres alfabeticos en un texto ASCII.
 * Esta diseado para usarse como herramienta en la decodificacion de textos
 * codificados (encriptados) con el mtodo de desplazamiento Csar (Caesar shift)
 * 
 * Existiendo ya otros programas similares, este reconoce vocales acentuadas y la "N"
 * con tilde; esto con el fin de que sea util en la decodificacion de textos en
 * espaol.
 * 
 * Los datos obtenidos pueden ser almacenados, en forma binaria (o, mediante una redireccion,
 * en formato ASCII).
 *
 * Autor: T. Nahual (Raza-Mexicana)
 *
 * Uso: lestat -t | -b archivo_entrada [archivo_salida]
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include "lestat.h"


int main(int argc, char* argv[])
{

  unsigned short results[strlen(alpha)];  // Espacio reservado para resultados
  unsigned long chartot=0;  // Total de caracteres alfabeticos

  FILE* fptr;       // Indice del archivo
  char* buffer;     // Indice de texto
  char* alphaptr;   // Indice de caracteres base
  char fchar;       // Caracteres individuales en el texto
  char alphasav;    // Variable de respaldo
  float frec;       // Variable de frecuencia (porcentaje)
  int i=0;          // Variable de conteo general
  int j;            // Otra variable
  char c;           // Variable de opciones
  struct stat info; // Estructura que contiene informacion de archivo
  record_t stats;

  if (argc < 3) {

    uso(argv[0]);
    exit (EXIT_FAILURE);

  }

  c = getopt(argc, argv, "tb");

  if ( c == 't' ) {

    fptr=openfile (argv[2], "r");

    /* Obtener informacion del archivo de texto, reservar memoria para el texto que contiene */
    stat(argv[2],&info);
    buffer = (char*) malloc((info.st_size+1) * sizeof(char));

    /*
      Libc solia tener soporte para operaciones utilizando ISO-8859-1. 
      Para evitar problemas entre versiones antiguas y recientes, utilizar
      las siguientes funciones.
      
      Todo character es convertido a mayusculas, para simplificar las cosas.
    */
#ifdef ISO_8859_1
    eurochar(fptr, buffer, fchar);
    
#else
    stdchar(fptr, buffer, fchar);

#endif

    fclose(fptr);

    /* Indicar caracteres base *
     * Procesar conteo de caracteres e imprimir resultados *
     */

    alphaptr=alpha;

    for (i=0;i<strlen(alpha);i++) {
      
      alphasav=alphaptr[1];
      alphaptr[1]='\0';
      results[i]=alphacount(buffer, alphaptr);
      chartot+=results[i];
      alphaptr[1]=alphasav;
      alphaptr++;

    }

#ifdef DESP_TXT

    printf("\nEl texto en el archivo es:\n%s\n\n", buffer);

#endif

    puts("\nResultados de Analisis Literal\n_____________________________________");
    printf("Tamao del archivo: %d\n", info.st_size);
    printf("Total de caracteres alfabeticos: %d\n\n", chartot);
    printf("Letra\tNmero\tFrecuencia(%)\tGrafico(%)");
    
    if ( argv[3] != '\0' ) {
      fptr = openfile (argv[3], "w+b");
    }
    
    for (i=0;i<strlen(alpha);i++) {
      frec=(float) 100*results[i]/chartot;

      printf("\n%c\t%d\t%.2f\t\t|", alpha[i], results[i], frec);
      for (j=0; j < (int) frec; j++) printf("|");

      stats.ltr=alpha[i];
      stats.num=results[i];
      stats.frec=frec;

      if ( argv[3] != '\0' ) {
	fwrite(&stats, sizeof(record_t), 1, fptr);
      }

    }

    puts("");
    puts("");

    free(buffer);


    if ( argv[3] != '\0' ) {
      printf("Los resultados fueron almacenados en \"%s\"\n\n", argv[3]);
      fclose(fptr);
    }

  } else if ( c == 'b' ) {

    fptr = openfile (argv[2], "r+b");

      printf("\nResultados de Analisis Literal:\n_______________________________\n\n");
      printf("Letra\tNmero\tFrecuencia(%)\n\n");
 
      while(fread(&stats, sizeof(record_t), 1, fptr)) {
	printf("%c\t%d\t%.2f\n", stats.ltr, stats.num, stats.frec);
      }

      printf("\n_______________________________\n\n");

    fclose(fptr);

  } else {

    printf("Opcion invalida: \"%s\"", argv[1]);
    uso(argv[0]);
    exit(EXIT_FAILURE);

  }
 
  exit (EXIT_SUCCESS);

}









